import base64
from copy import deepcopy
from io import BytesIO
from math import floor
from time import localtime, strftime
from typing import Tuple

import matplotlib.pyplot as plt
from matplotlib import font_manager as fm
from numpy import average
from PIL import Image, ImageDraw, ImageFont

from .__meta__ import getMeta

localDir = getMeta("localDir")
gachaTypeDict = getMeta("gachaTypeDict")
pieFontPath = localDir + "LXGW-Bold-minipie.ttf"
pilFontPath = localDir + "LXGW-{}.ttf"
# TEXTSTR = [
#   "鸦羽弓以理服人飞天御剑黎明神铁影阔沐浴龙血的诺艾尔冷刃翡玉法球弹魔导绪论"
#   "砂糖西风猎黑缨枪讨英杰谭凝光祭礼大匣里灭辰射手之誓迪奥娜北斗安柏琴达利亚"
#   "吟行秋笛班尼特莫七钟流浪乐章雷泽昭心长菲谢芭拉空残丽莎雨裁绝弦四原典重云"
#   "秘刻晴阿斯鹰藏辛焱离香菱贯虹槊凯翼卢克无工脊和璞鸢狼末路傲卷贝多斫峰甘",
# ]


# 返回百分数字符串 / 根据百分数返回颜色
def percent(a: int, b: int, rt: str = "") -> str:
    if not rt:
        return str(round(a / b * 100, 2)) + "%"
    # 由概率生成颜色
    # https://github.com/voderl/genshin-gacha-analyzer/blob/main/src/pages/ShowPage/AnalysisChart/utils.ts
    percentColors = [
        {"pct": 0.0, "color": {"r": 46, "g": 200, "b": 5}},
        {"pct": 0.77, "color": {"r": 67, "g": 93, "b": 250}},
        {"pct": 1.0, "color": {"r": 255, "g": 0, "b": 0}},
    ]
    pct = a / b
    for level in percentColors:
        if pct < level["pct"]:
            prevKey = percentColors.index(level) - 1
            prevCl = percentColors[prevKey]["color"]
            prevPct = percentColors[prevKey]["pct"]
            upPct = (pct - prevPct) / (level["pct"] - prevPct)
            lowPct = 1 - upPct
            nowCl = level["color"]
            clR = floor(lowPct * prevCl["r"] + upPct * nowCl["r"])
            clG = floor(lowPct * prevCl["g"] + upPct * nowCl["g"])
            clB = floor(lowPct * prevCl["b"] + upPct * nowCl["b"])
            return "#{:02X}{:02X}{:02X}".format(clR, clG, clB)
            # return "rgb({},{},{})".format(clR, clG, clB)
    return "#FF5652"


# 设置 Pillow 绘制字体
def fs(size: int, width: str = "Bold"):
    return ImageFont.truetype(pilFontPath.format(width), size=size)


# 转换 Image 对象图片为 Base64 编码字符串
def img2Base64(pic: Image.Image) -> str:
    buf = BytesIO()
    pic.save(buf, format="PNG", quality=100)
    base64_str = base64.b64encode(buf.getbuffer()).decode()
    return "base64://" + base64_str


# 逐行绘制不超过宽度限制的彩色五星历史记录文字
async def colorfulFive(star5Data: list, fontSize: int, maxWidth: int,
                       isWeapon: bool = False) -> Image.Image:
    ImageSize = (maxWidth, 400)
    coordX, coordY = 0, 0
    fontPadding = 10
    text1st = "五星历史记录："
    indent1st, fH = fs(fontSize).getsize(text1st)
    spaceW = indent1st / len(text1st)
    stepY = fH + fontPadding
    result = Image.new("RGBA", ImageSize, "#f9f9f9")
    tDraw = ImageDraw.Draw(result)
    tDraw.text((coordX, coordY), text1st, font=fs(fontSize), fill="black")
    coordX += indent1st
    for item in star5Data:
        color = percent(item["count"], 80 if isWeapon else 90, "color")
        for word in [item["name"], f"[{item['count']}]"]:
            wordW = fs(fontSize).getsize(word)[0]
            if coordX + wordW <= maxWidth:
                tDraw.text((coordX, coordY),
                           word, font=fs(fontSize), fill=color)
                coordX += wordW + spaceW / 2
            elif word[0] == "[":
                coordX, coordY = 0, (coordY + stepY)
                tDraw.text((coordX, coordY),
                           word, font=fs(fontSize), fill=color)
                coordX = wordW + spaceW
            else:
                aval = int((maxWidth - coordX) / spaceW)
                splitStr = [item["name"][:aval], item["name"][aval:]]
                for i in range(len(splitStr)):
                    s = splitStr[i]
                    tDraw.text((coordX, coordY),
                               s, font=fs(fontSize), fill=color)
                    if i == 0:
                        coordX, coordY = 0, (coordY + stepY)
                    else:
                        partW = fs(fontSize).getsize(s)[0]
                        coordX = partW + spaceW / 2
    coordY += stepY + fontPadding * 2
    star5Avg = average([item["count"] for item in star5Data])
    tDraw.text((0, coordY), "五星平均抽数：", font=fs(fontSize), fill="black")
    tDraw.text((indent1st, coordY), f"{star5Avg:.2f}", font=fs(fontSize),
               fill=percent(round(star5Avg), 80 if isWeapon else 90, "color"))
    result = result.crop((0, 0, maxWidth, coordY + fH))
    return result


# 提取统计数据
async def calcStat(gachaLogs: dict) -> dict:
    single = {
        "total": 0,
        "cntNot5": 0,
        "cntStar3": 0,
        "cntChar4": 0,
        "cntWeapon4": 0,
        "char5": [],
        "weapon5": [],
        "startTime": "",
        "endTime": "",
    }
    # 重排顺序为 301 302 200 100（角色、武器、常驻、新手
    selfType = sorted(gachaTypeDict.keys(), key=lambda k: k[0], reverse=True)
    stat = {gachaTypeId: deepcopy(single) for gachaTypeId in selfType}
    for gachaType in gachaLogs:
        counter = 0
        pityCounter = 0
        gachaStat = stat[gachaType]
        gachaList = gachaLogs[gachaType]
        gachaList.reverse()
        # !dont gachaList.sort(key=lambda item: item["time"], reverse=False)
        for item in gachaList:
            counter = counter + 1
            pityCounter = pityCounter + 1
            itemName = item["name"]
            itemType = item["item_type"]
            rankType = int(item["rank_type"])
            gachaStat["total"] = counter
            if rankType == 5:
                t = "char5" if itemType == "角色" else "weapon5"
                gachaStat[t].append({"name": itemName, "count": pityCounter})
                pityCounter = 0
            elif rankType == 4:
                t = "cntChar4" if itemType == "角色" else "cntWeapon4"
                gachaStat[t] = gachaStat[t] + 1
            else:
                gachaStat["cntStar3"] = gachaStat["cntStar3"] + 1
        wp5Cnts = [gachaStat["weapon5"][n]["count"]
                   for n in range(len(gachaStat["weapon5"]))]
        char5Cnts = [gachaStat["char5"][n]["count"]
                     for n in range(len(gachaStat["char5"]))]
        gachaStat["cntNot5"] = counter - sum(wp5Cnts) - sum(char5Cnts)
        if len(gachaList):
            timeNow = strftime("%Y-%m-%d %H:%M:%S", localtime())
            gachaStat["startTime"] = gachaList[0].get("time", timeNow)
            gachaStat["endTime"] = gachaList[-1].get("time", timeNow)
    return stat


# 绘制单个饼图
async def drawPie(stat: dict, rt: str = "") -> Tuple[str, Image.Image, bool]:
    partMap = [
        {"label": "三星武器", "color": "#73c0de", "total": stat["cntStar3"]},
        {"label": "四星武器", "color": "#91cc75", "total": stat["cntWeapon4"]},
        {"label": "四星角色", "color": "#5470c6", "total": stat["cntChar4"]},
        {"label": "五星武器", "color": "#ee6666", "total": len(stat["weapon5"])},
        {"label": "五星角色", "color": "#fac858", "total": len(stat["char5"])},
    ]
    # 如果可显示项目多于 4 项，考虑隐藏三星数据
    showStar3 = True
    if len([p for p in partMap if p["total"]]) > 4:
        partMap = [p for p in partMap if p["label"] != "三星武器"]
        showStar3 = False
    # 提取数据
    labels = [p["label"] for p in partMap if p["total"]]
    colors = [p["color"] for p in partMap if p["total"]]
    sizes = [p["total"] for p in partMap if p["total"]]
    explode = [
        (0.05 if "五星" in p["label"] else 0)
        for p in partMap if p["total"]
    ]
    # 绘制饼图
    fig, ax = plt.subplots()
    fmProp = fm.FontProperties(fname=pieFontPath)
    txtProp = {"fontsize": 16, "fontproperties": fmProp}
    ax.set_facecolor("#f9f9f9")
    ax.pie(sizes, labels=labels, colors=colors, autopct='%0.2f%%',
           labeldistance=1.1, pctdistance=0.7, startangle=60, radius=0.7,
           explode=explode, shadow=False, textprops=txtProp)
    ax.axis("equal")
    plt.tight_layout()
    # 生成图片
    ioBytes = BytesIO()
    plt.savefig(ioBytes, format="png")
    ioBytes.seek(0)
    if rt == "base64":
        b64Code = base64.b64encode(ioBytes.read()).decode()
        return b64Code, Image.new("RGB", (0, 0)), showStar3
    else:
        return "", Image.open(ioBytes), showStar3


# 通过 PIL 和 matplotlib 绘制统计信息图片
async def gnrtGachaInfo(rawData: dict) -> str:
    wishStat = await calcStat(rawData["gachaLogs"])
    # print(wishStat)
    gotPool = [key for key in wishStat if wishStat[key]["total"] > 0]
    imageList = []
    for gachaType in gotPool:
        poolImg = Image.new("RGBA", (500, 1000), "#f9f9f9")
        tDraw = ImageDraw.Draw(poolImg)
        poolStat = wishStat[gachaType]
        poolName = gachaTypeDict[gachaType]
        isWeapon = True if gachaType == "302" else False
        # 绘制标题
        poolNameW, poolImgH = fs(30).getsize(poolName)
        tDraw.text((int((500 - poolNameW) / 2), 20),
                   poolName, font=fs(30), fill="black")
        poolImgH += 20 * 2
        # 绘制饼状图
        unuse, pieImg, showStar3 = await drawPie(poolStat)
        # pieImg.size = (640, 480)
        pieImg = pieImg.resize((500, int(pieImg.height * 500 / pieImg.width)),
                               Image.ANTIALIAS).convert("RGBA")
        poolImg.paste(pieImg, (0, poolImgH), pieImg)
        poolImgH += pieImg.height
        if not showStar3:
            tDraw.text((15, int(poolImgH - 35)),
                       "* 三星武器数据已隐藏", font=fs(20), fill="#808080")
        # 绘制抽卡时间
        startTime = poolStat["startTime"].split(" ")[0]
        endTime = poolStat["endTime"].split(" ")[0]
        timeStat = f"{startTime} ~ {endTime}"
        poolImgH += 10
        timeStatW, timeStatH = fs(20).getsize(timeStat)
        tDraw.text((int((500 - timeStatW) / 2), poolImgH),
                   timeStat, font=fs(20), fill="#808080")
        poolImgH += timeStatH + 20
        # 绘制抽数统计
        poolTotal = poolStat["total"]
        notStar5 = poolStat["cntNot5"]
        totalStr1 = "共计 "
        totalStr2 = str(poolTotal)
        totalStr3 = " 抽"
        totalStr4 = "，已累计 "
        totalStr5 = str(notStar5)
        totalStr6 = " 抽未出 5 星"
        startW = 20
        tDraw.text((startW, poolImgH), totalStr1,
                   font=fs(25), fill="black")
        startW += fs(25).getsize(totalStr1)[0]
        tDraw.text((startW, poolImgH), totalStr2,
                   font=fs(25), fill="rgb(24,144,255)")
        startW += fs(25).getsize(totalStr2)[0]
        tDraw.text((startW, poolImgH), totalStr3,
                   font=fs(25), fill="black")
        if notStar5 > 0 and poolName != "新手祈愿":
            pityCnt = 80 if isWeapon else 90
            notStar5Color = percent(notStar5, pityCnt, "color")
            startW += fs(25).getsize(totalStr3)[0]
            tDraw.text((startW, poolImgH), totalStr4,
                       font=fs(25), fill="black")
            startW += fs(25).getsize(totalStr4)[0]
            tDraw.text((startW, poolImgH), totalStr5,
                       font=fs(25), fill=notStar5Color)
            startW += fs(25).getsize(totalStr5)[0]  # "rgb(47,192,22)"
            tDraw.text((startW, poolImgH), totalStr6,
                       font=fs(25), fill="black")
        poolImgH += fs(25).getsize(totalStr1)[1] + 20 * 2
        # 绘制概率统计
        totalStar = {
            "五星": {
                "cnt": len(poolStat["weapon5"]) + len(poolStat["char5"]),
                "color": "#C0713D"
            },
            "四星": {
                "cnt": poolStat["cntWeapon4"] + poolStat["cntChar4"],
                "color": "#A65FE2"
            },
            "三星": {
                "cnt": poolStat["cntStar3"],
                "color": "#4D8DF7"
            },
        }
        probList = [
            {"level": key, "total": value["cnt"], "color": value["color"]}
            for key, value in totalStar.items() if value["cnt"] > 0
        ]
        for item in probList:
            cntStr = f"{item['level']}：{item['total']} 次"
            probStr = f"[{item['total'] / poolTotal * 100:.2f}%]"
            tDraw.text((20, poolImgH), cntStr, font=fs(25), fill=item["color"])
            probStrW = fs(25).getsize(probStr)[0]
            tDraw.text((int(400 - probStrW), poolImgH), probStr,
                       font=fs(25), fill=item["color"])
            poolImgH += fs(25).getsize(cntStr)[1] + 20
        # 绘制五星物品统计
        poolImgH += 20
        star5Data = [*poolStat["char5"], *poolStat["weapon5"]]
        if star5Data:
            statPic = await colorfulFive(star5Data, 25, 460, isWeapon)
            statPicW, statPicH = statPic.size
            statPicCoord = (int((500 - statPicW) / 2), poolImgH)
            poolImg.paste(statPic, statPicCoord, statPic)
            poolImgH += statPicH
        # 绘制完成
        poolImgH += 20
        poolImg = poolImg.crop((0, 0, 500, poolImgH))
        imageList.append(poolImg)
    # 合并图片
    maxHeight = max([img.height for img in imageList])
    resultImg = Image.new("RGBA", (500 * len(imageList), maxHeight), "#f9f9f9")
    for i, img in enumerate(imageList):
        resultImg.paste(img, (500 * i, 0), img)
    # resultImg.save("/home/ubuntu/paimon/resources/gacha/result.png", "PNG")
    # bgImg = Image.open(f"pic.png")
    # result.paste(bgImg, (coordX, coordY), bgImg)
    resImgB64 = img2Base64(resultImg)
    return resImgB64


# [unuse] 通过 Playwright 和 matplotlib 绘制统计信息图片
async def gnrtGachaInfoHtml(rawData: dict) -> str:
    wishStat = await calcStat(rawData["gachaLogs"])
    htmlTplPath = f"{localDir}htmlTpl.html"
    with open(htmlTplPath, "r", encoding="utf-8") as f:
        htmlTpl = str(f.read())
    infoHtml = """"""
    gotPool = [key for key in wishStat if wishStat[key]["total"] > 0]
    for gachaType in gotPool:
        poolStat = wishStat[gachaType]
        poolName = gachaTypeDict[gachaType]
        poolTotal = poolStat["total"]
        # 数据饼图与抽卡时间
        chartB64, unuse, showStar3 = await drawPie(poolStat, rt="base64")
        startTime = poolStat["startTime"]
        endTime = poolStat["endTime"]
        timeStat = f"{startTime} ~ {endTime}"
        # 未出五星抽数提示
        notStar5 = poolStat["cntNot5"]
        notStar5Str = f"，已累计 {notStar5} 抽未出 5 星"
        totalStat = f"共计 {poolTotal} 抽{notStar5Str if notStar5 else ''}"
        # 抽数统计与概率生成
        totalStar = {
            "五星": len(poolStat["weapon5"]) + len(poolStat["char5"]),
            "四星": poolStat["cntWeapon4"] + poolStat["cntChar4"],
            "三星": poolStat["cntStar3"],
        }
        probList = [
            f"{key}：{value} <span>[{percent(value, poolTotal)}]</span>"
            for key, value in totalStar.items() if value > 0
        ]
        probStat = "<br />".join(probList)
        # 五星物品历史记录
        star5Data = [*poolStat["char5"], *poolStat["weapon5"]]
        if star5Data:
            star5List = [f"{item['name']}[{item['count']}]"
                         for item in star5Data]
            star5Stat = "五星历史记录：" + "·".join(star5List)
            star5Avg = average([item["count"] for item in star5Data])
            star5Stat += f"<br />五星平均抽数：{star5Avg:.2f}"
        else:
            star5Stat = "五星物品正在路上..."
        # 替换模板
        poolHtml = f"""        <div class="pool">
            <div class="pool-name">{poolName}</div>
            <div class="pool-chart">
                <img src="data:image/png;base64,{chartB64}" />
                <span>{timeStat}</span>
            </div>
            <div class="pool-comment">
                <p>{totalStat}</p>
                <p>{probStat}</p>
                <p>{star5Stat}</p>
            </div>
        </div>\n"""
        # print(poolHtml)
        infoHtml += poolHtml
    newHtmlPath = htmlTplPath.replace("htmlTpl", rawData["uid"])
    with open(newHtmlPath, "w", encoding="utf-8") as f:
        f.write(htmlTpl.replace("    {{info}}", infoHtml))
    # TODO Plawright 截图
    return newHtmlPath



import json
import os
from time import time

from .__meta__ import getMeta
from .data_source import cacheData, mergeData

localDir = getMeta("localDir")


async def importUIGF(qq: str, uigfData: dict) -> dict:
    rawData = {
        "uid": uigfData["info"]["uid"],
        "time": uigfData["info"].get("export_timestamp", int(time())),
        "url": "",
        "gachaLogs": {
            "100": [],
            "200": [],
            "301": [],
            "302": [],
        }
    }
    rt = {"msg": "", "data": rawData}
    uigfData["list"].reverse()
    for item in uigfData["list"]:
        gachaType = item.get("gacha_type", "")
        uigfType = item.get("uigf_gacha_type", "")
        item.update({
            "uid": item.get("uid", rawData["uid"]),
            "gacha_type": gachaType if gachaType != "" else uigfType,
            "item_id": item.get("item_id", ""),
            "count": item.get("count", "1"),
            "time": item["time"],
            "name": item["name"],
            "lang": item.get("lang", "zh-cn"),
            "item_type": item["item_type"],
            "rank_type": item["rank_type"],
            "id": item["id"],
        })
        item.pop("uigf_gacha_type")
        nowType = item["gacha_type"]
        gachaTypeId = nowType if nowType != "400" else "301"
        rawData["gachaLogs"][gachaTypeId].append(item)
    import2File = f"{localDir}cache-{rawData['uid']}.json"
    # 如果已有本地记录尝试合并
    if os.path.isfile(import2File):
        backupFile = import2File.replace(".json", "-last.json")
        with open(import2File, "r", encoding="utf-8") as f:
            existData = json.load(f)
        with open(backupFile, "w", encoding="utf-8") as f:
            json.dump(existData, f, ensure_ascii=False, indent=2)
        wait2Merge = {"msg": "正在导入新的数据..", "data": existData}
        rt = await mergeData(wait2Merge, rt, qq, fw=False)
    # 写入本地缓存
    await cacheData(qq, rt["data"])
    return rt




import json
import os
import re
import sys
from asyncio import sleep as asleep
from time import time
from urllib import parse

from httpx import AsyncClient
from nonebot.log import logger

from .__meta__ import getMeta

localDir = getMeta("localDir")
expireSec = getMeta("expireSec")
basicUrl = getMeta("basicUrl")
basicUrlOversea = getMeta("basicUrlOversea")
gachaTypeDict = getMeta("gachaTypeDict")


# 检查抽卡记录链接是否失效 [httpx]
# 返回值：str
#   "成功" / 错误信息
async def checkAuthKey(url: str) -> str:
    try:
        async with AsyncClient() as client:
            res = await client.get(url)
            resJson = res.json()
        # with open(f"{localDir}checkAuthKey-{int(time.time())}.json", "w",
        #           encoding="utf-8") as f:
        #     json.dump(resJson, f, ensure_ascii=False, indent=2)
    except Exception as e:
        logger.error("抽卡记录 API 请求解析出错："
                     + str(sys.exc_info()[0]) + "\n" + str(e))
        return "检查链接时效性出错\n[{}]".format(str(sys.exc_info()[0]))
    if not resJson["data"]:
        return "链接 AuthKey 可能失效\n[{}]".format(resJson["message"])
    return "成功"


# 检查及提取抽卡记录链接 [checkAuthKey]
# 返回值：str
#   抽卡记录链接 / 错误信息
async def checkLogUrl(logUrl: str) -> str:
    if not logUrl:
        return "请至少给我一次抽卡记录链接！"
    logUrl = logUrl.replace("&amp;", "&")
    # 含有关键词视为正确链接
    if "getGachaLog" in logUrl:
        url = logUrl
    else:
        # 提取输入字符中的抽卡记录链接并转换
        matchUrl = re.search("hk4e/event/.*#/log", logUrl)
        if not matchUrl:
            return "未找到有效的抽卡记录链接！"
        url = f"https://webstatic.mihoyo.com/{matchUrl.group(0)}"
        splitUrl = url.split("?")
        if "webstatic-sea" in splitUrl[0] or "hk4e-api-os" in splitUrl[0]:
            splitUrl[0] = basicUrlOversea
        else:
            splitUrl[0] = basicUrl
        url = "?".join(splitUrl)
    # 检查抽卡记录链接是否有效
    keyStatus = await checkAuthKey(url)
    if keyStatus != "成功":
        return keyStatus
    return url


# 获取本地缓存抽卡记录 [fileread]
# 返回值：dict
#   msg: "" / 错误信息
#   data: 抽卡记录数据 / {"time": int}
async def getCacheData(qq: str) -> dict:
    cache = {"msg": "", "data": {}}
    cacheFile = localDir + "cache-config.json"
    # 本地无缓存配置文件时，创建缓存配置文件
    if not os.path.isfile(cacheFile):
        with open(cacheFile, "w", encoding="utf-8") as f:
            json.dump({}, f, ensure_ascii=False, indent=2)
        cache["msg"] = "暂无本地抽卡记录！"
        cache["data"]["time"] = int(time()) - expireSec - 1
        return cache
    with open(cacheFile, "r", encoding="utf-8") as f:
        cacheConfig = json.load(f)
    # 本地有缓存配置文件时，检查是否有用户缓存
    if qq not in cacheConfig.keys():
        cache["msg"] = "暂无本地抽卡记录！"
        cache["data"]["time"] = int(time()) - expireSec - 1
        return cache
    # 本地有用户缓存时读取缓存的抽卡记录
    with open(cacheConfig[qq], "r", encoding="utf-8") as f:
        cachedRawData = json.load(f)
    cache["data"] = cachedRawData
    return cache


# 获取指定类型抽卡记录单页链接
# 返回值：str "https://..."
def getGachaLogsApi(url: str, page: int, typeId: str, endId: str) -> str:
    parsed = parse.urlparse(url)
    querys = parse.parse_qsl(str(parsed.query))
    paramDict = dict(querys)
    paramDict.update({
        "size": "20",  # 限制单页最大获取总抽数 20
        "gacha_type": typeId,
        "page": str(page),
        "lang": "zh-cn",
        "end_id": endId,
    })
    param = parse.urlencode(paramDict)
    path = str(url).split("?")[0]
    api = path + "?" + param
    return api


# 获取指定类型抽卡记录 [getGachaLogsApi]
# 返回值：list [{}, {}, ...]
async def getGachaList(url: str, gachaTypeId: str) -> list:
    gachaList, endId, page = [], "0", 1
    for pageTryCnt in range(1, 9999):
        logger.debug(f"正在获取 {gachaTypeDict[gachaTypeId]} 第 {page} 页")
        api = getGachaLogsApi(url, page, gachaTypeId, endId)
        gachaListPage = []
        async with AsyncClient() as client:
            res = await client.get(api)
            resJson = res.json()
        # with open(f"{localDir}getGachaList-{gachaTypeId}-{page}.json", "w",
        #           encoding="utf-8") as f:
        #     json.dump(resJson, f, ensure_ascii=False, indent=2)
        if resJson["data"]:
            gachaListPage = resJson["data"].get("list", [])
        elif resJson["message"] == "visit too frequently":
            # 访问过于频繁，等待 5s
            await asleep(5)
            continue
        if not gachaListPage:
            break
        for item in gachaListPage:
            gachaList.append(item)
        # 更新下一页用于生成链接的数据
        endId = resJson["data"]["list"][-1]["id"]
        page += 1
        await asleep(1)
    return gachaList


# 获取全部类型抽卡记录 [checkLogUrl] [getGachaList]
# 返回值：dict
#   msg: "" / 错误信息
#   data: 抽卡记录数据 / {"uid": "", ...}
async def getRawData(logUrl: str, force: bool = False) -> dict:
    raw = {"msg": "" if not force else "强制获取最新数据..", "data": {}}
    # 检查链接是否失效
    logUrlChecked = await checkLogUrl(logUrl)
    if "getGachaLog" not in logUrlChecked:
        raw["msg"] += logUrlChecked
        return raw
    # 获取最新抽卡记录
    uidGot = False
    gachaData = {
        "uid": "",
        "time": "",
        "url": logUrlChecked,
        "gachaLogs": {}
    }
    for gachaType in gachaTypeDict:
        gachaList = await getGachaList(logUrlChecked, gachaType)
        gachaData["gachaLogs"][gachaType] = gachaList
        # 从记录中获取一次 UID
        if gachaList and not uidGot:
            gachaData["uid"] = gachaList[0]["uid"]
            uidGot = True
    gachaData["time"] = int(time())
    raw["data"] = gachaData
    if not uidGot:
        raw["msg"] = "获取最新抽卡记录失败！"
    return raw


# 缓存数据 [filewrite]
async def cacheData(qq: str, rawData: dict) -> None:
    uid = rawData["uid"]
    # 创建 UID 对应缓存文件
    cacheFile = localDir + f"cache-{uid}.json"
    with open(cacheFile, "w", encoding="utf-8") as f:
        json.dump(rawData, f, ensure_ascii=False, indent=2)
    # 更新用于 getCacheData 的缓存配置文件
    cgfFile = localDir + "cache-config.json"
    with open(cgfFile, "r", encoding="utf-8") as f:
        cacheConfig = json.load(f)
    cacheConfig[qq] = cacheFile
    with open(cgfFile, "w", encoding="utf-8") as f:
        json.dump(cacheConfig, f, ensure_ascii=False, indent=2)


# 合并数据 [cacheData]
# 返回值：dict
#   msg: 合并消息
#   data: 合并数据
async def mergeData(cache: dict, raw: dict, qq: str, fw: bool = True) -> dict:
    # 若无新增数据，则直接返回缓存数据
    if not raw["data"].get("gachaLogs", {}):
        cache["msg"] += raw["msg"]
        return cache
    # 若无缓存数据，则直接返回新增数据
    if not cache["data"].get("gachaLogs", {}):
        raw["msg"] = cache["msg"] + raw["msg"]
        if raw["data"].get("gachaLogs", {}):
            await cacheData(qq, raw["data"])
        return raw
    # 既有缓存数据又有新增数据，执行合并
    locData = cache["data"]
    newData = raw["data"]
    msgList = []
    merged = {"msg": "", "data": {}}
    for gachaType in gachaTypeDict:
        logsLoc = locData["gachaLogs"][gachaType]
        logsNew = newData["gachaLogs"][gachaType]
        # 新增数据与缓存数据不同时合并
        if logsNew != logsLoc:
            tempList = []
            itemsGot = [[got["time"], got["name"]] for got in logsLoc]
            for i in range(len(logsNew)):
                item = [logsNew[i]["time"], logsNew[i]["name"]]
                if item not in itemsGot:
                    tempList.insert(0, logsNew[i])
                else:
                    pass
            for item in tempList:
                locData["gachaLogs"][gachaType].insert(0, item)
            if tempList:
                s = f"新增 {len(tempList)} 条{gachaTypeDict[gachaType]}记录.."
                msgList.append(s)
        else:
            pass
    # 处理附加信息
    locData["uid"] = newData["uid"]
    locData["time"] = newData["time"]
    locData["url"] = newData["url"]
    # 缓存合并数据并生成结果
    if fw:
        await cacheData(qq, locData)
    merged["data"] = locData
    merged["msg"] = "\n".join(msgList)
    if cache["msg"] or raw["msg"]:
        merged["msg"] = cache["msg"] + raw["msg"] + "\n" + merged["msg"]
    return merged


# 获取抽卡记录入口 [getCacheData] [getRawData] [mergeData]
# 返回值：dict
#   msg: "" / 合并消息 / 错误信息
#   data: 抽卡记录数据 / 合并数据 / {"": "", ...}
async def getGachaData(qq: str, logUrl: str = "",
                       cache: dict = {}, force: bool = False) -> dict:
    # 读取缓存
    cache = await getCacheData(qq) if not cache else cache
    now = int(time())
    # 缓存未过期且未要求强制刷新，返回缓存数据
    if now - cache["data"]["time"] < expireSec and not force:
        return cache
    if force and not logUrl:
        cache["msg"] += "（强制刷新不可用！"
        return cache
    # 无链接且要求强制刷新，返回缓存数据
    logUrl = logUrl if logUrl else cache["data"].get("url", "")
    # 刷新数据
    raw = await getRawData(logUrl, force=force)
    # 合并数据
    fullData = await mergeData(cache, raw, qq)
    return fullData



import os
import sys

from nonebot import on_command
from nonebot.exception import FinishedException
from nonebot.log import logger
from nonebot.typing import T_State

try:
    from nonebot.adapters.onebot.v11 import (Bot, GroupMessageEvent, Message,
                                             MessageEvent, MessageSegment,
                                             PrivateMessageEvent)
except ImportError:
    from nonebot.adapters.cqhttp import Bot, Message, MessageSegment
    from nonebot.adapters.cqhttp.event import (GroupMessageEvent, MessageEvent,
                                               PrivateMessageEvent)

from .data_export import exportGacha
from .data_render import gnrtGachaInfo
from .data_source import checkLogUrl, getCacheData, getGachaData
from .upload_cos import initCosClient, uploadFile

cosClient = initCosClient()

gMatcher = on_command("抽卡记录", aliases={"ckjl"}, priority=1)
eMatcher = on_command("抽卡记录导出", aliases={"gcexport", "ckjldc"}, priority=1)
# iMatcher = on_command("抽卡记录导入", aliases={"gcimport", "ckjldr"}, priority=1)
# fMatcher = on_notice("offline_file", priority=100)


@gMatcher.handle()
async def gachaHistory(bot: Bot, event: MessageEvent, state: T_State):
    q = event.get_user_id()
    s = event.get_plaintext().strip()
    c = await getCacheData(q)
    # 群聊用户不在缓存中且没有输入链接，结束会话
    if c["msg"] == "暂无本地抽卡记录！":
        if isinstance(event, GroupMessageEvent) and s in ["", "-f", "--force"]:
            c["msg"] += "请在私聊中使用「抽卡记录」命令添加抽卡记录链接！"
            await gMatcher.finish(Message(c["msg"]))
    # 检测到缓存的用户将缓存数据传递至 got 方法进一步判断
    else:
        state["cache"] = c
        state["url"] = c["data"]["url"]
    # 只在触发时判定用户是否要求强制刷新
    state["force"] = True if s in ["-f", "--force"] else False
    # 传递附加在首次触发命令的消息中的链接
    if s not in ["", "-f", "--force"]:
        state["url"] = s


# Message.template("{next}")
@gMatcher.got("url", prompt=("派蒙将从接下来你回复的内容中找出有效链接用于统计：\n\n"
                             "* webstatic.mihoyo.com/hk4e/.../#/log 在内即可"))
async def gachaHistoryRes(bot: Bot, event: MessageEvent, state: T_State):
    qq = event.get_user_id()
    # 已缓存用户特殊处理
    if "cache" not in state.keys():
        state["cache"] = {}
    else:
        # 不是缓存中的链接，则判定为需要强制刷新
        if state["url"] != state["cache"]["data"]["url"]:
            # 粗略判断输入是否合法，不合法继续使用缓存链接
            if "https" not in state["url"]:
                state["url"] = ""
            else:
                state["force"] = True
        # 检查缓存中的链接是否失效，发送提示信息
        else:
            expireCheck = await checkLogUrl(state["cache"]["data"]["url"])
            state["url"] = ""
            if "https" not in expireCheck:
                warnMag = "缓存的链接状态异常！回复链接以更新 / 回复其他内容忽略"
                await gMatcher.reject(Message(warnMag))
    # 常规流程，获取数据、生成图片、发送消息
    rt = await getGachaData(qq, state["url"], state["cache"], state["force"])
    if rt["msg"]:
        await gMatcher.send(Message(rt["msg"]))
    if not rt["data"].get("gachaLogs", ""):
        raise FinishedException
    try:
        imgB64 = await gnrtGachaInfo(rt["data"])
        await gMatcher.finish(MessageSegment.image(imgB64))
    except FinishedException:
        pass
    except Exception as e:
        logger.error("抽卡记录图片生成出错：" + str(sys.exc_info()[0]) + "\n" + str(e))
        await bot.send_private_msg(
            message=(f"[error] [genshin_export]\n由用户 {event.get_user_id()} 触发"
                     f"\n{str(sys.exc_info()[0])}\n{str(e)}"),
            user_id=int(list(bot.config.superusers)[0]),
        )


@eMatcher.handle()
async def gachaExport(bot: Bot, event: MessageEvent, state: T_State):
    s = [i.strip() for i in event.get_plaintext().split(" ")]
    q = event.get_user_id()
    # 处理输入的导出参数
    if not len(s):
        exUser = q
        outFmt = "xlsx"
    elif len(s) == 1:
        exUser = s[0] if s[0].isdigit() else q
        outFmt = "xlsx" if s[0].isdigit() else s[0]
    elif len(s) == 2:
        exUser = s[0] if s[0].isdigit() else s[1]
        outFmt = s[1] if s[0].isdigit() else s[0]
    else:
        await eMatcher.finish("指令格式错误！")
    if exUser != q and q not in bot.config.superusers:
        await eMatcher.finish("你没有权限导出该用户抽卡记录！")
    c = await getCacheData(exUser)
    # 用户不在缓存中，结束会话
    if c["msg"] == "暂无本地抽卡记录！":
        c["msg"] += "请在私聊中使用「抽卡记录」命令添加抽卡记录链接！"
        await gMatcher.finish(Message(c["msg"]))
    # 生成文件并尝试发送
    exData = await exportGacha(c["data"], outFmt)
    if not exData["file"]:
        await eMatcher.finish(Message(exData["msg"]))
    elif isinstance(event, GroupMessageEvent):
        try:
            await bot.upload_group_file(
                group_id=event.group_id,
                file=exData["file"],
                name=exData["file"].split(os.sep)[-1],
            )
        except Exception as e:
            logger.error(
                "发送抽卡记录导出群文件失败："
                + str(sys.exc_info()[0]) + "\n" + str(e)
            )
            exData["msg"] += "但是群文件上传失败，稍后再试吧！"
    elif isinstance(event, PrivateMessageEvent):
        if not cosClient:
            exData["msg"] += "但是因为未成功初始化腾讯云对象存储，无法发送文件！"
        else:
            exFileUrl = await uploadFile(cosClient, exData["file"])
            if exFileUrl:
                exData["msg"] += "派蒙已将文件上传至腾讯云 COS，有效期 1 小时，"
                exData["msg"] += f"请及时下载保存：\n{exFileUrl}"
            else:
                exData["msg"] += "但是上传文件失败，请稍后再试！"
    # 删除本地下载文件
    await eMatcher.send(Message(exData["msg"]))
    try:
        os.remove(exData["file"])
    except Exception:
        pass


# @iMatcher.handle()
# async def gachaImport(bot: Bot, event: MessageEvent, state: T_State):



import os
from nonebot import get_driver

try:
    resPath = get_driver().config.resources_dir
    localDir = f"{resPath}{os.sep}gachalogs{os.sep}"
    assert os.path.exists(localDir)
except (AssertionError, AttributeError):
    resPath = os.path.dirname(os.path.abspath(__file__))
    localDir = f"{resPath}{os.sep}gachalogs{os.sep}"
    if not os.path.exists(localDir):
        os.makedirs(localDir)

try:
    expireSec = get_driver().config.gacha_expire_sec
    assert isinstance(expireSec, int)
except (AssertionError, AttributeError):
    expireSec = 60 * 60


def getMeta(need: str):
    gachaTypeDontIndex = ["400"]
    gachaTypeDictFull = {
        "100": "新手祈愿",
        "200": "常驻祈愿",
        "301": "角色活动祈愿",
        "302": "武器活动祈愿",
        "400": "角色活动祈愿-2",
    }
    gachaTypeDict = {
      key: value for key, value in gachaTypeDictFull.items()
      if key not in gachaTypeDontIndex
    }
    basicUrl = "https://hk4e-api.mihoyo.com/event/gacha_info/api/getGachaLog"
    basicUrlOversea = basicUrl.replace("hk4e-api", "hk4e-api-os")
    gachaMeta = {
        "gachaTypeDictFull": gachaTypeDictFull,
        "gachaTypeDict": gachaTypeDict,
        "basicUrl": basicUrl,
        "basicUrlOversea": basicUrlOversea,
        "localDir": localDir,
        "expireSec": expireSec,
    }
    return gachaMeta[need]


def getCOSMeta() -> dict:
    cfg = get_driver().config
    Name = cfg.cos_bucket_name if hasattr(cfg, "cos_bucket_name") else ""
    Region = cfg.cos_bucket_region if hasattr(cfg, "cos_bucket_region") else ""
    secretId = cfg.cos_secret_id if hasattr(cfg, "cos_secret_id") else ""
    secretKey = cfg.cos_secret_key if hasattr(cfg, "cos_secret_key") else ""
    cosMeta = {
        "bucketName": Name,
        "bucketRegion": Region,
        "secretId": secretId,
        "secretKey": secretKey,
    }
    return cosMeta



import json
from time import localtime, strftime, time

from xlsxwriter import Workbook

from .__meta__ import getMeta

localDir = getMeta("localDir")
gachaTypeDict = getMeta("gachaTypeDict")
gachaTypeDictFull = getMeta("gachaTypeDictFull")


# 生成物品 ID
def gnrtId():
    id = 1000000000000000000
    while True:
        id = id + 1
        yield str(id)


# 转换原始请求结果为 UIGF JSON
async def transUIGF(uid: str, gachaLogs: dict) -> dict:
    UIGF = {
        "info": {
            "uid": uid,
            "lang": "zh-cn",
            "export_time": strftime("%Y-%m-%d %H:%M:%S", localtime()),
            "export_timestamp": int(time()),
            "export_app": "nonebot-genshin-gacha-export",
            "export_app_version": "v0.1.0",
            "uigf_version": "v2.2"
        },
        "list": []
    }
    # 写入数据
    for gachaType in gachaTypeDict:
        gachaLog = gachaLogs.get(gachaType, [])
        gachaLog = sorted(gachaLog, key=lambda i: i["time"], reverse=False)
        for item in gachaLog:
            item["uigf_gacha_type"] = gachaType
        UIGF["list"].extend(gachaLog)
    UIGF["list"] = sorted(UIGF["list"], key=lambda item: item["time"])
    # 缺失物品 ID 补充
    id = gnrtId()
    for item in UIGF["list"]:
        if item.get("id", "") == "":
            item["id"] = next(id)
    UIGF["list"] = sorted(UIGF["list"], key=lambda item: item["id"])
    # 返回 UIGF JSON
    return UIGF


# 转换原始请求结果为 UIGF XLSX
async def transXLSX(uid: str, gachaLogs: dict, uigfList: list) -> str:
    exportTime = strftime("%Y%m%d%H%M%S", localtime())
    wbPath = f"{localDir}Wish-{uid}-{exportTime}.xlsx"
    wb = Workbook(wbPath)
    # 重排顺序为 301 302 200 100（角色、武器、常驻、新手
    # sorted(gachaTypeDict.keys(), key=lambda item: item[0], reverse=True)
    gachaTypeList = ["301", "302", "200", "100"]
    for gachaType in gachaTypeList:
        # 新建页面和样式
        worksheet = wb.add_worksheet(gachaTypeDict[gachaType])
        headerStyle = wb.add_format({
            "align": "left",
            "font_name": "微软雅黑",
            "color": "#757575",
            "bg_color": "#dbd7d3",
            "border_color": "#c4c2bf",
            "border": 1,
            "bold": True
        })
        contentStyle = wb.add_format({
            "align": "left",
            "font_name": "微软雅黑",
            "border_color": "#c4c2bf",
            "bg_color": "#ebebeb",
            "border": 1
        })
        star5Style = wb.add_format({"color": "#bd6932", "bold": True})
        star4Style = wb.add_format({"color": "#a256e1", "bold": True})
        star3Style = wb.add_format({"color": "#8e8e8e"})
        worksheet.set_column("A:A", 22)
        worksheet.set_column("B:B", 14)
        worksheet.set_column("E:E", 14)
        # 表头
        header = ["时间", "名称", "物品类型", "星级", "祈愿类型", "总次数", "保底内"]
        worksheet.write_row(0, 0, header, headerStyle)
        worksheet.freeze_panes(1, 0)
        # 从最旧的数据开始写入
        counter = 0
        pityCounter = 0
        gachaList = gachaLogs[gachaType]
        gachaList.reverse()
        for item in gachaList:
            timeStr = item["time"]
            itemName = item["name"]
            itemType = item["item_type"]
            rankType = int(item["rank_type"])
            gachaType = item["gacha_type"]
            gachaTypeName = gachaTypeDictFull.get(gachaType, "")
            counter = counter + 1
            pityCounter = pityCounter + 1
            content = [
                timeStr, itemName, itemType, rankType,
                gachaTypeName, counter, pityCounter
            ]
            worksheet.write_row(counter, 0, content, contentStyle)
            if content[3] == 5:
                pityCounter = 0
        # 三星、四星、五星物品高亮
        contentRow1st, contentCol1st = 1, 0
        contentRowLast, contentColLast = len(gachaList), len(header) - 1
        worksheet.conditional_format(
            contentRow1st, contentCol1st, contentRowLast, contentColLast,
            {"type": "formula", "criteria": "=$D2=3", "format": star3Style}
        )
        worksheet.conditional_format(
            contentRow1st, contentCol1st, contentRowLast, contentColLast,
            {"type": "formula", "criteria": "=$D2=4", "format": star4Style}
        )
        worksheet.conditional_format(
            contentRow1st, contentCol1st, contentRowLast, contentColLast,
            {"type": "formula", "criteria": "=$D2=5", "format": star5Style}
        )
    # 原始数据表
    worksheet = wb.add_worksheet("原始数据")
    rawHeader = [
        "count", "gacha_type", "id", "item_id", "item_type", "lang",
        "name", "rank_type", "time", "uid", "uigf_gacha_type"
    ]
    worksheet.write_row(0, 0, rawHeader)
    allCounter = 0
    for item in uigfList:
        count = item.get("count", "")
        gachaType = item.get("gacha_type", "")
        id = item.get("id", "")
        itemId = item.get("item_id", "")
        itemType = item.get("item_type", "")
        lang = item.get("lang", "")
        name = item.get("name", "")
        rankType = item.get("rank_type", "")
        timeStr = item.get("time", "")
        uigfUid = item.get("uid", "")
        uigfGachaType = item.get("uigf_gacha_type", "")
        rawContent = [
            count, gachaType, id, itemId, itemType, lang,
            name, rankType, timeStr, uigfUid, uigfGachaType
        ]
        worksheet.write_row(allCounter + 1, 0, rawContent)
        allCounter += 1
    # 关闭工作簿
    wb.close()
    # 返回文件路径
    return wbPath


# 导出抽卡数据
async def exportGacha(rawData: dict, outFormat: str) -> dict:
    rt = {"msg": "", "file": ""}
    # 无抽卡记录数据直接返回
    if not rawData.get("gachaLogs", ""):
        rt["msg"] = "没有抽卡记录可供导出哦！"
        return rt
    # 判断输出格式
    if outFormat.lower() in ["j", "json", "u", "uigf"]:
        outFormat = "uigf"
    else:
        # if outFormat.lower() in ["x", "xlsx", "excel"]:
        outFormat = "xlsx"
    uid = rawData["uid"]
    gachaLogs = rawData["gachaLogs"]
    # 转换原始数据为 UIGF 格式数据
    uigfData = await transUIGF(uid, gachaLogs)
    # 生成对应格式文件
    if outFormat == "xlsx":
        xlsxPath = await transXLSX(uid, gachaLogs, uigfData["list"])
        rt["msg"] = "抽卡记录导出 Excel 完成！"
        rt["file"] = xlsxPath
    elif outFormat == "uigf":
        exportTime = strftime("%Y%m%d%H%M%S", localtime())
        uigfPath = f"{localDir}UIGF-{uid}-{exportTime}.json"
        with open(uigfPath, "w", encoding="utf-8") as f:
            json.dump(uigfData, f, ensure_ascii=False, indent=2)
        rt["msg"] = "抽卡记录导出 JSON(UIGF) 完成！"
        rt["file"] = uigfPath
    else:
        rt["msg"] = "暂未支持的导出格式！"
    return rt



import hmac
import os
import sys
import time
from hashlib import sha1

from nonebot.log import logger

from .__meta__ import getCOSMeta

cosConfig = getCOSMeta()
bucketName = cosConfig["bucketName"]
bucketRegion = cosConfig["bucketRegion"]
secretId = cosConfig["secretId"]
secretKey = cosConfig["secretKey"]


# 计算 COS 文件链接签名表单字段
# https://cos5.cloud.tencent.com/static/cos-sign/
def getSignature(file: str) -> str:
    def toBytes(s: str) -> bytes:
        return s.encode("utf-8")
    # SignKey = HMAC-SHA1(SecretKey, [q-key-time])
    signTime = int(time.time()) - 60
    expireTime = signTime + 3600
    timeStr = f"{signTime};{expireTime}"
    signKey = hmac.new(toBytes(secretKey), toBytes(timeStr), sha1).hexdigest()
    # HttpString = [HttpMethod]\n[HttpURI]\n[HttpParameters]\n[HttpHeaders]\n
    httpString = u"get\n/{}\n\n\n".format(file)
    # StringToSign = [q-sign-algorithm]\n[q-sign-time]\nSHA1-HASH(HttpString)\n
    httpStrSha = sha1(toBytes(httpString)).hexdigest()
    strToSign = u"sha1\n{}\n{}\n".format(timeStr, httpStrSha)
    # Signature = HMAC-SHA1(SignKey,StringToSign)
    sign = hmac.new(toBytes(signKey), toBytes(strToSign), sha1).hexdigest()
    params = [
        "q-sign-algorithm=sha1",
        f"q-ak={secretId}",
        f"q-sign-time={timeStr}",
        f"q-key-time={timeStr}",
        "q-header-list=",
        "q-url-param-list=",
        f"q-signature={sign}",
    ]
    return "&".join(params)


# 初始化 COS 客户端，一个 Bucket 只需一个客户端即可
def initCosClient() -> object:
    if "" in cosConfig.values():
        return None
    try:
        from qcloud_cos import CosConfig, CosS3Client
    except ImportError:
        from pip._internal import main as pipmain
        pipmain(['install', "cos-python-sdk-v5"])
        from qcloud_cos import CosConfig, CosS3Client
    except Exception as e:
        logger.error(
            "安装 XML Python SDK 失败："
            + str(sys.exc_info()[0]) + "\n" + str(e)
        )
        return None
    try:
        config = CosConfig(
            Region=bucketRegion, Token=None, Scheme="https",
            SecretId=secretId, SecretKey=secretKey
        )
        client = CosS3Client(config)
        return client
    except Exception as e:
        logger.error(
            "初始化 cos client 失败："
            + str(sys.exc_info()[0]) + "\n" + str(e)
        )
        return None


# 上传文件到 COS
async def uploadFile(client: object, filePath: str) -> str:
    bucketDomain = f"https://{bucketName}.cos.{bucketRegion}.myqcloud.com"
    fileName = filePath.split(os.sep)[-1]
    with open(filePath, "rb") as fp:
        response = client.put_object(
            Bucket=bucketName,
            Body=fp,
            Key=fileName,
            StorageClass="STANDARD",
            EnableMD5=False
        )
    logger.debug("文件上传 Etag：" + response["ETag"])
    urlSigned = f"{bucketDomain}/{fileName}?{getSignature(fileName)}"
    return urlSigned



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['nonebot_plugin_gachalogs']

package_data = \
{'': ['*']}

install_requires = \
['httpx>=0.20.0,<1.0.0',
 'matplotlib>=3.5.1',
 'nonebot-adapter-onebot>=2.0.0b1',
 'nonebot2>=2.0.0a14',
 'xlsxwriter>=3.0.2']

setup_kwargs = {
    'name': 'nonebot-plugin-gachalogs',
    'version': '0.1.1',
    'description': 'A Genshin GachaLogs analysis plugin for Nonebot2',
    'long_description': '<h1 align="center">Nonebot Plugin GachaLogs</h1></br>\n\n\n<p align="center">🤖 用于统计及导出原神祈愿记录的 Nonebot2 插件</p></br>\n\n\n<p align="center">\n  <a href="https://github.com/monsterxcn/nonebot-plugin-gachalogs/actions">\n    <img src="https://img.shields.io/github/workflow/status/monsterxcn/nonebot-plugin-gachalogs/Build%20distributions?style=flat-square" alt="actions">\n  </a>\n  <a href="https://raw.githubusercontent.com/monsterxcn/nonebot-plugin-gachalogs/master/LICENSE">\n    <img src="https://img.shields.io/github/license/monsterxcn/nonebot-plugin-gachalogs?style=flat-square" alt="license">\n  </a>\n  <a href="https://pypi.python.org/pypi/nonebot-plugin-gachalogs">\n    <img src="https://img.shields.io/pypi/v/nonebot-plugin-gachalogs?style=flat-square" alt="pypi">\n  </a>\n  <img src="https://img.shields.io/badge/python-3.7.3+-blue?style=flat-square" alt="python"><br />\n</p></br>\n\n\n**安装方法**\n\n\n使用以下命令安装插件本体 ~~（插件暂未发布于 PyPI）~~：\n\n\n```bash\n# 从 Git 安装\ngit clone https://github.com/monsterxcn/nonebot-plugin-gachalogs.git\ncd nonebot_plugin_gachalogs\ncp -r nonebot_plugin_gachalogs /path/to/nonebot/plugins/\ncp -r resources/gachalogs /path/to/resources/\n```\n\n\n然后检查一下，别忘了安装依赖 `matplotlib` `Pillow` `xlsxwriter`！\n\n\n```bash\npython3 -m pip install matplotlib Pillow xlsxwriter\n```\n\n\n<details><summary><i>从 PyPI 安装</i></summary></br>\n\n\n```bash\n# 从 PyPI 安装\npython3 -m pip install nonebot-plugin-gachalogs\n```\n\n\n从 PyPI 安装后需要手动将 `resources/gachalogs` 文件夹内资源下载到服务端。\n\n\n</details>\n\n\n打开 Nonebot2 正在使用的 `.env` 文件，参考 [.env.example](.env.example) 添加以下配置：\n\n\n - `resources_dir` 包含 gachalogs 文件夹的上级目录路径\n - `gacha_expire_sec` 祈愿历史记录本地缓存过期秒数，不设置默认 1 小时\n - `cos_bucket_name` 腾讯云 COS 存储桶名称\n - `cos_bucket_region` 腾讯云 COS 存储桶地域\n - `cos_secret_id` 腾讯云 COS 存储桶 SecretId\n - `cos_secret_key` 腾讯云 COS 存储桶 SecretKey\n\n\n\\* *关于腾讯云 COS 的配置仅用于私聊导出文件*\n\n\n重启 Bot 即可体验此插件。\n\n\n**使用方法**\n\n\n插件支持以下命令：\n\n\n - `抽卡记录` / `ckjl`\n   \n   返回一张统计饼图，样式与 https://genshin.voderl.cn/ 一致。\n   \n   初次使用要求输入一次祈愿历史记录链接，只要回复的内容中含有即可，不必手动截取准确的链接地址。\n   \n   附带 `-f` / `--force` 可要求强制获取最新祈愿记录，祈愿记录结果默认缓存 1 小时。\n   \n   ![祈愿统计图](resources/readme/result.png)\n   \n - `抽卡记录导出` / `ckjldc`\n   \n   导出祈愿历史记录文件，可选格式包括 `.xlsx` 表格和 `.json` 文件，均符合 [统一可交换祈愿记录标准](https://github.com/DGP-Studio/Snap.Genshin/wiki/StandardFormat)（UIGF）格式。\n   \n   附带 `j` / `json` / `u` / `uigf` 可指定导出 `.json` 文件，否则默认导出 `.xlsx` 表格。\n   \n   管理员可使用 `ckjldc [qq] [format]` 格式命令导出指定 QQ 的祈愿历史记录。\n   \n   此功能需要配置腾讯云 COS 以实现私聊文件发送，创建的存储桶建议设为私有读写。\n   \n   ![导出示意图](resources/readme/export.png)\n\n\n**特别鸣谢**\n\n\n[@nonebot/nonebot2](https://github.com/nonebot/nonebot2/) | [@Mrs4s/go-cqhttp](https://github.com/Mrs4s/go-cqhttp) | [@sunfkny/genshin-gacha-export](https://github.com/sunfkny/genshin-gacha-export) | [@voderl/genshin-gacha-analyzer](https://github.com/voderl/genshin-gacha-analyzer)\n\n\n> 插件主要功能是从 [@sunfkny/genshin-gacha-export](https://github.com/sunfkny/genshin-gacha-export) 抄的，溜溜…\n',
    'author': 'monsterxcn',
    'author_email': 'monsterxcn@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/monsterxcn/nonebot-plugin-gachalogs',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.7.3,<4.0',
}


setup(**setup_kwargs)
